@@ -289,6 +289,16 @@ bitstream_putbits(bitstream_t *bs, unsigned long bits, int bit_width) {
     return 0;
 }
 
+int
+bitstream_putbits_signed(bitstream_t *bs, signed long bits, int bit_width) {
+    if (bits < 0) {
+        register signed long msb = 1 << (bit_width - 1);
+        register signed long bitmask = (2 * msb) - 1;
+        bits = (-bits  - 1) ^ bitmask;
+    }
+    return bitstream_putbits(bs, bits, bit_width);
+}
+
 unsigned long
 bitstream_getbits(bitstream_t *bs, int bit_width) {
     int i;
@@ -306,15 +316,11 @@ bitstream_getbits(bitstream_t *bs, int bit_width) {
 
 signed long
 bitstream_getbits_signed(bitstream_t *bs, int bit_width) {
-    int i;
-    int bit;
-    unsigned long bits = 0;
-    for (i=0 ; i < bit_width ; i++) {
-        bit = bitstream_getbit(bs);
-        if (bit == -1) {
-            return -1;
-        }
-        bits |= bit << (bit_width - 1 - i);
+    signed long bits = bitstream_getbits(bs, bit_width);
+    register signed long msb =  bits & (1 << (bit_width - 1));
+    if (msb) {
+        register signed long bitmask = (2 * msb) - 1;
+        bits = - (bits ^ bitmask) - 1;
     }
     return bits;
 }
@@ -383,7 +389,7 @@ bitstream_length(bitstream_t *bs) {
 
 
 
-extern signed long
+signed long
 bitstream_unsigned2signed(unsigned long num, int size) {
     unsigned long sig_bit = 1 << (size - 1);
     if ((sig_bit & num) == 0) {
@@ -394,7 +400,7 @@ bitstream_unsigned2signed(unsigned long num, int size) {
     }
 }
 
-extern unsigned long
+unsigned long
 bitstream_signed2unsigned(signed long num, int size) { // XXX check me!
     if (0 <= num){
         return (unsigned long) num;
@@ -404,6 +410,36 @@ bitstream_signed2unsigned(signed long num, int size) { // XXX check me!
         return - ((num^mask) & mask) - 1;
     }
 }
+
+int
+bitstream_need_bits_unsigned(unsigned long n) {
+    int i;
+    for (i = 0 ; n ; i++) {
+        n >>= 1;
+    }
+    return i;
+}
+
+int
+bitstream_need_bits_signed(signed long n) {
+    int i;
+    int ret;
+    if (n < -1) {
+        n = -1 - n;
+    }
+    if (n >= 0) {
+        for (i = 0 ; n ; i++) {
+            n >>= 1;
+        }
+        ret = 1 + i;
+        ;
+    } else { // n == -1
+        ret = 1;
+    }
+    return ret;
+}
+
+
 /*
  * error handling
  */
