@@ -34,6 +34,7 @@
 #include "swf_tag_sound.h"
 #include "swf_tag_action.h"
 #include "swf_tag_sprite.h"
+#include "swf_tag_shape.h"
 #include "swf_tag.h"
 #include "swf_object.h"
 
@@ -49,35 +50,40 @@ static int le_swfed;
  * Every user visible function must have an entry in swfed_functions[].
  */
 zend_function_entry swfed_functions[] = {
-	PHP_ME(swfed,  __construct, NULL, 0)
-   	PHP_ME(swfed,  input, NULL, 0)
-   	PHP_ME(swfed,  output, NULL, 0)
-   	PHP_ME(swfed,  getHeaderInfo, NULL, 0)
-       	PHP_ME(swfed,  setHeaderInfo, NULL, 0)
-      	PHP_ME(swfed,  getTagList, NULL, 0)
-   	PHP_ME(swfed,  getTagDetail, NULL, 0)
-       	PHP_ME(swfed,  getTagData, NULL, 0)
-        PHP_ME(swfed,  replaceTagData, NULL, 0)
-        PHP_ME(swfed,  setShapeAdjustMode, NULL, 0)
-   	PHP_ME(swfed,  getJpegData, NULL, 0)
-   	PHP_ME(swfed,  getJpegAlpha, NULL, 0)
-   	PHP_ME(swfed,  replaceJpegData, NULL, 0)
-        PHP_ME(swfed,  getPNGData, NULL, 0)
-        PHP_ME(swfed,  replacePNGData, NULL, 0)
-        PHP_ME(swfed,  replaceGIFData, NULL, 0)
-        PHP_ME(swfed,  applyShapeMatrixFactor, NULL, 0)
-        PHP_ME(swfed,  applyShapeRectFactor, NULL, 0)
-        PHP_ME(swfed,  getSoundData, NULL, 0)
-        PHP_ME(swfed,  replaceMLDData, NULL, 0)
-   	PHP_ME(swfed,  getEditString, NULL, 0)
-   	PHP_ME(swfed,  replaceEditString, NULL, 0)
-   	PHP_ME(swfed,  getActionData, NULL, 0)
-       	PHP_ME(swfed,  disasmActionData, NULL, 0)
-        PHP_ME(swfed,  swfInfo, NULL, 0)
-	{NULL, NULL, NULL}	/* Must be the last line in swfed_functions[] */
+    PHP_ME(swfed,  __construct, NULL, 0)
+    PHP_ME(swfed,  input, NULL, 0)
+    PHP_ME(swfed,  output, NULL, 0)
+    PHP_ME(swfed,  getHeaderInfo, NULL, 0)
+    PHP_ME(swfed,  setHeaderInfo, NULL, 0)
+    PHP_ME(swfed,  getTagList, NULL, 0)
+    PHP_ME(swfed,  getTagDetail, NULL, 0)
+    PHP_ME(swfed,  getTagData, NULL, 0)
+    PHP_ME(swfed,  replaceTagData, NULL, 0)
+    PHP_ME(swfed,  setShapeAdjustMode, NULL, 0)
+    PHP_ME(swfed,  getShapeIdListByBitmapRef, NULL, 0)
+    PHP_ME(swfed,  getBitmapSize, NULL, 0)
+    PHP_ME(swfed,  getJpegData, NULL, 0)
+    PHP_ME(swfed,  getJpegAlpha, NULL, 0)
+    PHP_ME(swfed,  replaceJpegData, NULL, 0)
+    PHP_ME(swfed,  getPNGData, NULL, 0)
+    PHP_ME(swfed,  replacePNGData, NULL, 0)
+    PHP_ME(swfed,  replaceGIFData, NULL, 0)
+    PHP_ME(swfed,  applyShapeMatrixFactor, NULL, 0)
+    PHP_ME(swfed,  applyShapeRectFactor, NULL, 0)
+    PHP_ME(swfed,  getSoundData, NULL, 0)
+    PHP_ME(swfed,  replaceMLDData, NULL, 0)
+    PHP_ME(swfed,  getEditString, NULL, 0)
+    PHP_ME(swfed,  replaceEditString, NULL, 0)
+    PHP_ME(swfed,  getActionData, NULL, 0)
+    PHP_ME(swfed,  disasmActionData, NULL, 0)
+//    PHP_ME(swfed,  insertActionSetVariables, NULL, 0)
+    PHP_ME(swfed,  swfInfo, NULL, 0)
+    {NULL, NULL, NULL}	/* Must be the last line in swfed_functions[] */
 };
 /* }}} */
 
+#define SWFED_VERSION "0.24"
+
 /* {{{ swfed_module_entry
  */
 zend_module_entry swfed_module_entry = {
@@ -92,7 +98,7 @@ zend_module_entry swfed_module_entry = {
 	PHP_RSHUTDOWN(swfed),	/* Replace with NULL if there's nothing to do at request end */
 	PHP_MINFO(swfed),
 #if ZEND_MODULE_API_NO >= 20010901
-	"0.23", /* Replace with version number for your extension */
+	SWFED_VERSION, /* Replace with version number for your extension */
 #endif
 	STANDARD_MODULE_PROPERTIES
 };
@@ -190,6 +196,7 @@ PHP_MINFO_FUNCTION(swfed)
 	int png_support = 0;
 	php_info_print_table_start();
 	php_info_print_table_header(2, "SWF Editor support", "enabled");
+        php_info_print_table_row(2, "SWF Editor version", SWFED_VERSION);
 #ifdef HAVE_PNG
 	png_support = 1;
 #endif
@@ -535,6 +542,48 @@ PHP_METHOD(swfed, setShapeAdjustMode) {
     swf_object_set_shape_adjust_mode(swf, mode);
     RETURN_TRUE;
 }
+
+PHP_METHOD(swfed, getShapeIdListByBitmapRef) {
+    long bitmap_id;
+    swf_object_t *swf = NULL;
+    swf_tag_t *tag = NULL;
+    int i;
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
+                              "l", &bitmap_id) == FAILURE) {
+        RETURN_FALSE;
+    }
+    swf = get_swf_object(getThis() TSRMLS_CC);
+    array_init(return_value);
+    i = 0;
+    for (tag = swf->tag ; tag ; tag=tag->next) {
+        register int tag_code = tag->tag;
+        if (isShapeTag(tag_code) && (swf_tag_shape_bitmap_identity(tag, bitmap_id) == 0)) {
+            swf_tag_shape_detail_t *swf_tag_shape = tag->detail;
+            add_index_long(return_value, i, (long) swf_tag_shape->shape_id);
+            i++;
+        }
+    }
+}
+
+PHP_METHOD(swfed, getBitmapSize) {
+    long bitmap_id;
+    swf_object_t *swf = NULL;
+    int width, height;
+    int ret;
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
+                              "l", &bitmap_id) == FAILURE) {
+        RETURN_FALSE;
+    }
+    swf = get_swf_object(getThis() TSRMLS_CC);
+    ret = swf_object_get_bitmap_size(swf, bitmap_id, &width, &height);
+    if (ret) {
+        RETURN_FALSE;
+    }
+    array_init(return_value);
+    add_assoc_long(return_value, "width", (long) width);
+    add_assoc_long(return_value, "height", (long) height);
+}
+
 PHP_METHOD(swfed, getJpegData) {
     unsigned long image_id = 0;
     unsigned long len = 0;
@@ -912,6 +961,11 @@ PHP_METHOD(swfed, disasmActionData) {
         }
     }
     swf_action_list_destroy(action_list);
+    RETURN_TRUE;
+}
+
+PHP_METHOD(swfed, insertActionSetVariables) {
+    RETURN_TRUE;
 }
 
 PHP_METHOD(swfed, swfInfo) {
